#include <gsl/gsl_sf.h>
#include <cmath>

#include "../math/mellin_Dk.hh"
#include "../parameters.hh"
#include "gammaLO.hh"


// LO anomalous dimensions gamma_0 
// defined by gamma = alpha_s gamma_0 + ...



dcomplex gamma0qq(dcomplex N) {
  return CF/2./M_PI * ( 1./N
			- 1./(N+1.)
			- 2.*psi0(N+1.) - 2.*EulerGamma
			+ 3./2. );
}
dcomplex gamma0qg(dcomplex N, int nf) {
  return nf/2./M_PI * ( 2.+N+N*N ) / N/(N+1.)/(N+2.);
}
dcomplex gamma0gq(dcomplex N) {
  return CF/2./M_PI * ( 2.+N+N*N ) / N/(N*N-1.);
}
dcomplex gamma0gg(dcomplex N, int nf) {
  return CA/M_PI * ( 11./12.
		     + 1./(N-1.)
		     - 1./N
		     + 1./(N+1.)
		     - 1./(N+2.)
		     - psi0(N+1.) - EulerGamma )
    - nf/6./M_PI;
}
dcomplex gamma0plus(dcomplex N, int nf) {
  if(nf == 0) return gamma0gg(N,nf);
  dcomplex halftrace = (gamma0qq(N) + gamma0gg(N,nf))/2.;
  dcomplex det = gamma0qq(N) * gamma0gg(N,nf)  -  gamma0qg(N,nf) * gamma0gq(N);
  dcomplex res = halftrace + sqrt( halftrace*halftrace - det );
  return res;
}

// first derivatives
dcomplex gamma0qq_prime(dcomplex N) {
  return CF/2./M_PI * ( -1./N/N
			+ 1./(N+1.)/(N+1.)
			- 2.*psi_n(1,N+1.)
			);
}
dcomplex gamma0qg_prime(dcomplex N, int nf) {
  return nf/2./M_PI * ( - 1./N/N
			+ 2./(N+1.)/(N+1.)
			- 2./(N+2.)/(N+2.)
			);
}
dcomplex gamma0gq_prime(dcomplex N) {
  return CF/2./M_PI * ( - 2./(N-1.)/(N-1.)
			+ 2./N/N
			- 1./(N+1.)/(N+1.)
			);
}
dcomplex gamma0gg_prime(dcomplex N) {
  return CA/M_PI * ( -1./(N-1.)/(N-1.)
		     + 1./N/N
		     - 1./(N+1.)/(N+1.)
		     + 1./(N+2.)/(N+2.)
		     - psi_n(1,N+1.) );
}

// second derivatives
dcomplex gamma0qq_second(dcomplex N) {
  return CF/2./M_PI * ( 2./N/N/N
			- 2./(N+1.)/(N+1.)/(N+1.)
			- 2.*psi_n(2,N+1.)
			);
}
dcomplex gamma0qg_second(dcomplex N, int nf) {
  return nf/2./M_PI * ( 2./N/N/N
			- 4./(N+1.)/(N+1.)/(N+1.)
			+ 4./(N+2.)/(N+2.)/(N+2.)
			);
}
dcomplex gamma0gq_second(dcomplex N) {
  return CF/2./M_PI * ( 4./(N-1.)/(N-1.)/(N-1.)
			- 4./N/N/N
			+ 2./(N+1.)/(N+1.)/(N+1.)
			);
}
dcomplex gamma0gg_second(dcomplex N) {
  return CA/M_PI * ( 2./(N-1.)/(N-1.)/(N-1.)
		     - 2./N/N/N
		     + 2./(N+1.)/(N+1.)/(N+1.)
		     - 2./(N+2.)/(N+2.)/(N+2.)
		     - psi_n(2,N+1.) );
}







// real versions
////////////////
double gamma0qq(double N) {
  return CF/2./M_PI * ( 1./N
			- 1./(N+1.)
			- 2.*gsl_sf_psi(N+1.) - 2.*EulerGamma
			+ 3./2. );
}
double gamma0qg(double N, int nf) {
  return nf/2./M_PI * ( 2.+N+N*N ) / N/(N+1.)/(N+2.);
}
double gamma0gq(double N) {
  return CF/2./M_PI * ( 2.+N+N*N ) / N/(N*N-1.);
}
double gamma0gg(double N, int nf) {
  return CA/M_PI * ( 11./12.
		     + 1./(N-1.)
		     - 1./N
		     + 1./(N+1.)
		     - 1./(N+2.)
		     - gsl_sf_psi(N+1.) - EulerGamma )
    - nf/6./M_PI;
}
double gamma0plus(double N,int nf) {
  if(nf == 0) return gamma0gg(N,nf);
  double halftrace = (gamma0qq(N) + gamma0gg(N,nf))/2.;
  double det = gamma0qq(N) * gamma0gg(N,nf)  -  gamma0qg(N,nf) * gamma0gq(N);
  double res = halftrace + sqrt( halftrace*halftrace - det );
  return res;
}

// derivatives
double gamma0qq_prime(double N) {
  return CF/2./M_PI * ( -1./N/N
			+ 1./(N+1.)/(N+1.)
			- 2.*gsl_sf_psi_1(N+1.)
			);
}
double gamma0qg_prime(double N, int nf) {
  return nf/2./M_PI * ( - 1./N/N
			+ 2./(N+1.)/(N+1.)
			- 2./(N+2.)/(N+2.)
			);
}
double gamma0gq_prime(double N) {
  return CF/2./M_PI * ( - 2./(N-1.)/(N-1.)
			+ 2./N/N
			- 1./(N+1.)/(N+1.)
			);
}
double gamma0gg_prime(double N) {
  return CA/M_PI * ( -1./(N-1.)/(N-1.)
		     + 1./N/N
		     - 1./(N+1.)/(N+1.)
		     + 1./(N+2.)/(N+2.)
		     - gsl_sf_psi_1(N+1.) );
}

dcomplex SF_integrand(dcomplex N, double log1ox, int nf, dcomplex gam(dcomplex, int)) {
  return exp( N*log1ox ) * gam(N,nf) / M_PI;
}
double SF(double x, int nf, dcomplex gam(dcomplex, int)) { // Talbot algorithm
  double log1ox = -log(x);
  double res = 0.; 
  //int M = 66;
  int M = 36;
  //int M = int(26.-4.*log10(x));
  double r = 1.+2.*M/5./log1ox;
  double t, cott, sigma;
  dcomplex N;
  res = real(0.5 * SF_integrand(r,log1ox,nf, gam));
  for(int k=1; k<M; k++) {
    t = k*M_PI/M;
    cott = 1./tan(t);
    sigma = t + cott*(t*cott - 1.);
    N = r*t*(cott+I);
    res += real(SF_integrand(N,log1ox,nf,gam) * (1.+I*sigma));
  }
  res *= M_PI*r/M;
  return res;
}
double P0plus(double x, int nf) {
  return SF(x, nf, gamma0plus);
}
double P0gg(double x, int nf) {
  return SF(x, nf, gamma0gg);
}
